;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                 LCD-Routinen                ;;
;;                 ============                ;;
;;              (c)andreas-s@web.de            ;;
;;                                             ;;
;; 4bit-Interface                              ;;
;; DB4-DB7:       PD4-PD7                      ;;
;; RS:            PB1                          ;;
;; E:             PB0                          ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



 ;sendet ein Datenbyte an das LCD
lcd_data:
           push temp2
		   push temp3
		
		   mov temp2, temp1
           andi temp2, 0b11110000
		   sbi PORTB, PinRS       			;unteres Nibble auf Null setzen
           in temp3, PORTD
		   andi temp3,0b00001111
		   or temp3,temp2
		   out PORTD, temp3
		
           rcall lcd_enable             ;Enable-Routine aufrufen
		                                ;2. Nibble, kein swap da es schon

		   swap temp1                            ;an der richtigen stelle ist
           andi temp1, 0b11110000       ;obere Hlfte auf Null setzen
           sbi PORTB, PinRS	              				;entspricht 0b00010000
           in temp3, PORTD
		   andi temp3,0b00001111
		   or temp3,temp1
		   out PORTD, temp3
		
           rcall lcd_enable             ;Enable-Routine aufrufen
           rcall delay37us

		   inc LCD_CursorPos
			
		   pop temp3
		   pop temp2              ;Delay-Routine aufrufen
           ret                          ;zurck zum Hauptprogramm

 ;sendet einen Befehl an das LCD
lcd_command:                            ;wie lcd_data, nur ohne RS zu setzen
           push temp2
		   push temp3
		
		   mov temp2, temp1
           andi temp2, 0b11110000
		
		   cbi PORTB,PinRS
		   in temp3,PORTD
		   andi temp3,0b00001111
		   or temp3,temp2
           out PORTD, temp3
		
           rcall lcd_enable
		
		   swap temp1
           andi temp1, 0b11110000

		   cbi PORTB,PinRS
		   in temp3,PORTD
		   andi temp3,0b00001111
		   or temp3,temp1
           out PORTD, temp3
           rcall lcd_enable
           rcall delay37us

			
			pop temp3
			pop temp2

           ret

 ;erzeugt den Enable-Puls
lcd_enable:
           sbi PORTB, PinEn                 ;Enable high 		
		   rcall delay450ns              ;3 Taktzyklen warten
           cbi PORTB, PinEn                 ;Enable wieder low
		   ret                          ;Und wieder zurck

 ;Pause nach jeder bertragung

delay37us:
		   push temp1                             ;37us Pause
           ldi  temp1, 89
delay37us_:dec  temp1
           brne delay37us_
		   pop temp1
           ret                          ;wieder zurck

delay450ns:
		   push temp1                             ;450ns Pause
           ldi  temp1, 30
           delay450ns_:dec  temp1
           brne delay450ns_
		   pop temp1
           ret
                     ;wieder zurck

 ;Lngere Pause fr manche Befehle
delay5ms:                               ;5ms Pause
           push temp2
		   push temp1
		   ldi  temp1, $C8
L0:   ldi  temp2, $C8
L1:   dec  temp2
           brne L1
           dec  temp1
           brne L0
		   pop temp1
		   pop temp2
           ret

 ;Initialisierung: muss ganz am Anfang des Programms aufgerufen werden
lcd_init:
           ldi	temp4,01          ;50
powerupwait:
           rcall	delay5ms			;Startup Time warten
           dec	temp4
           brne	powerupwait

           ldi temp1, 0b00110000        ;muss 3mal hintereinander gesendet
		   out PORTD, temp1             ;werden zur Initialisierung
           rcall lcd_enable             ;1
           rcall delay5ms
		   rcall lcd_enable             ;2
           rcall delay5ms
		   rcall lcd_enable             ;und 3!
           rcall delay5ms
           ldi temp1, 0b00100000        ;4bit-Modus einstellen
           out PORTD, temp1
           rcall lcd_enable
           rcall delay5ms
           ldi temp1, 0b00101000        ;noch was einstellen...
           rcall lcd_command
           ldi temp1, 0b00001100        ;...nochwas...
           rcall lcd_command
           ldi temp1, 0b00000100        ;endlich fertig
           rcall lcd_command
           ret

 ;Sendet den Befehl zur Lschung des Displays
lcd_clear:
           push temp1
		   ldi temp1, 0b00000001   ;Display lschen
           rcall lcd_command
           rcall delay5ms
		   ldi LCD_CursorPos,0x80
		   pop temp1
           ret

CursorOn:	
			ldi temp1,0b00001111
			rcall lcd_command
			ret

CursorOff:	
			ldi temp1,0b00001100
			rcall lcd_command
			ret




Zeile1:
           push temp1
		   ldi temp1, 0b10000000   ;Display auf RAM-Adr. 00 stellen
           mov LCD_CursorPos,temp1
		   rcall lcd_command
		
		   pop temp1
           ret


Zeile2:
           push temp1
		   ldi temp1, 0b11000000   ;Display auf RAM-Adr. 0x40 stellen
           mov LCD_CursorPos,temp1
		   rcall lcd_command
			
		   pop temp1
           ret
Zeile3:
           push temp1
		   ldi temp1, 0b10010100   ;Display auf RAM-Adr. 0x14 stellen
           mov LCD_CursorPos,temp1
		   rcall lcd_command
			
		   pop temp1
           ret

Zeile4:
           push temp1
		   ldi temp1, 0b11010100   ;Display auf RAM-Adr. 0x54 stellen
           mov LCD_CursorPos,temp1
		   rcall lcd_command
			
		   pop temp1
           ret

;*******************************************************************************

AusgDisplay:
		
		
		sts SSCursorPositionEingabe,Lcd_CursorPos
		rcall CursorOff
		
		
		
		lds temp1,SSZeilenZa
		
		cpi temp1,0
		breq AusgDisplayZ1
		cpi temp1,1
		breq AusgDisplayZ2
		cpi temp1,2
		breq AusgDisplayZ3
		cpi temp1,3
		breq AusgDisplayZ4
		rjmp EndeAusgDisplay
		
AusgDisplayZ1:		
		rcall Zeile1
	   rcall AusgRAMZ1
	   rjmp EndeAusgDisplay

AusgDisplayZ2:		
		rcall Zeile2
	   rcall AusgRAMZ2
	   rjmp EndeAusgDisplay

AusgDisplayZ3:
	   rcall Zeile3
	   rcall AusgRAMZ3
	   rjmp EndeAusgDisplay

AusgDisplayZ4:	
		rcall Zeile4
	   rcall AusgRAMZ4
	   rjmp EndeAusgDisplay	

EndeAusgDisplay:
		lds temp1,SSZeilenZa
		inc temp1
		cpi temp1,4
		in temp2,SREG
		sbrc temp2,Z
		clr temp1
		sts SSZeilenZa,temp1		
		
		lds temp1,SSBitByte1
		sbrc temp1,BitMEinstEingabe               ;wenn auf Mode Eingabe gestellt, dann Cursorblinkend
		rcall AusgCursorPosition
		sbrc temp1,BitMParaEingabe
		rcall AusgCursorPosition
		ret
;*******************************************************************************

AusgRAMZ1:						
		ldi yl,low(SSLcdRAMZ1)
		ldi yh,high(SSLcdRAMZ1)		
		rjmp AusgZeichen
		
AusgRAMZ2:						
		ldi yl,low(SSLcdRAMZ2)
		ldi yh,high(SSLcdRAMZ2)
      rjmp AusgZeichen

AusgRAMZ3:			
		ldi yl,low(SSLcdRAMZ3)
		ldi yh,high(SSLcdRAMZ3)
      rjmp AusgZeichen

AusgRAMZ4:						
		ldi yl,low(SSLcdRAMZ4)
		ldi yh,high(SSLcdRAMZ4)
      rjmp AusgZeichen

AusgZeichen:
      ldi temp4,20	
loop_msg:
		ld	temp1,Y+									
		rcall	lcd_data			
		dec	temp4				
		brne	loop_msg		
		ret


;**********************************************************************************


AnzSign3:
		SBRS rBin1L,7
		rjmp AnzPosZahl3
NegZahl3:
		neg rBin1L
		ldi temp4,' '
		st Y+,temp4				
		ldi temp4,'-'
		st Y+,temp4
		ldi zl,SSBerRetL
		ldi zh,0
		rcall Bin2ToAsc5		
		ld temp1,z+
		ld temp1,z+
		ld temp1,z+		
		ld temp1,z+		
		ld temp1,z+
		st Y+,temp1
		
		ret

;***********************************************************************************		

AnzPosZahlOVNU2:
		
				
		ldi zl,SSBerRetL
		ldi zh,0
		rcall Bin2ToAsc5		
		ld temp1,z+
		ld temp1,z+
		ld temp1,z+		
		ld temp1,z+
		cpi temp1,' '
		in temp3,SREG
		sbrc temp3,Z
		ldi temp1,'0'		
		st Y+,temp1
		ld temp1,z+
		st Y+,temp1
		
		ret

;***********************************************************************************		

AnzPosZahl2:				
		ldi zl,SSBerRetL
		ldi zh,0
		rcall Bin2ToAsc5		
		ld temp1,z+
		ld temp1,z+
		ld temp1,z+		
		ld temp1,z+
		st Y+,temp1
		ld temp1,z+
		st Y+,temp1
		
		ret

;************************************************************************************
		
AnzPosZahl3:				
		ldi zl,SSBerRetL
		ldi zh,0
		rcall Bin2ToAsc5		
		ld temp1,z+
		ld temp1,z+
		ld temp1,z+
		st y+,temp1
		ld temp1,z+
		st Y+,temp1
		ld temp1,z+
		st Y+,temp1
		
		ret

;***********************************************************************************		

AnzPosZahl4:				
		ldi zl,SSBerRetL
		ldi zh,0
		rcall Bin2ToAsc5		
		ld temp1,z+		
		ld temp1,z+
		st y+,temp1
		ld temp1,z+
		st y+,temp1
		ld temp1,z+
		st Y+,temp1
		ld temp1,z+
		st Y+,temp1
		
		ret

;***********************************************************************************		
		
AnzPosZahl5:				
		ldi zl,SSBerRetL
		ldi zh,0
		rcall Bin2ToAsc5		
		ld temp1,z+
		st y+,temp1
		ld temp1,z+
		st y+,temp1
		ld temp1,z+
		st y+,temp1
		ld temp1,z+
		st Y+,temp1
		ld temp1,z+
		st Y+,temp1
		
		ret

;***********************************************************************************		

AnzPosKommaZahl3:

		ldi zl,SSBerRetL
		ldi zh,0
		rcall Bin2ToAsc5		
		ld temp1,z+
		ld temp1,z+
		ld temp1,z+
		
		st y+,temp1
		ld temp1,z+
		
		cpi temp1,' '
		in temp2,SREG
		sbrc temp2,Z
		ldi temp1,'0'		
		st Y+,temp1
		
		ldi temp1,','
		st Y+,temp1
		
		ld temp1,z+
		st Y+,temp1
		
		ret

;**********************************************************************************

AnzModus:		
		lpm temp1,z+
		st y+,temp1
      dec temp4
      brne AnzModus
      ret














